/* -*-c-*-
 *
 * $Id: alloc.c,v 1.3 1999/05/06 19:51:35 mdw Exp $
 *
 * Memory allocation functions
 *
 * (c) 1998 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the mLib utilities library.
 *
 * mLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * mLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with mLib; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------*
 *
 * $Log: alloc.c,v $
 * Revision 1.3  1999/05/06 19:51:35  mdw
 * Reformatted the LGPL notice a little bit.
 *
 * Revision 1.2  1999/05/05 18:50:31  mdw
 * Change licensing conditions to LGPL.
 *
 * Revision 1.1.1.1  1998/06/17 23:44:42  mdw
 * Initial version of mLib
 *
 */

/*----- Header files ------------------------------------------------------*/

/* --- ANSI headers --- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* --- Local headers --- */

#include "alloc.h"
#include "exc.h"
#include "track.h"

/*----- Functions provided ------------------------------------------------*/

/* --- @xmalloc@ --- *
 *
 * Arguments:	@size_t sz@ = size of block to allocate
 *
 * Returns:	Pointer to allocated block.
 *
 * Use:		Allocates memory.  If there's not enough memory, the
 *		exception @EXC_NOMEM@ is thrown.
 */

void *xmalloc(size_t sz)
{
  void *p = malloc(sz);
  if (!p)
    THROW(EXC_NOMEM);
  return (p);
}

/* --- @xstrdup@ --- *
 *
 * Arguments:	@const char *s@ = pointer to a string
 *
 * Returns:	Pointer to a copy of the string.
 *
 * Use:		Copies a string (like @strdup@ would, if it existed).  If
 *		there's not enough memory, the exception @EXC_NOMEM@ is
 *		thrown.
 */

char *xstrdup(const char *s)
{
  size_t sz = strlen(s) + 1;
  char *p = xmalloc(sz);
  memcpy(p, s, sz);
  return (p);
}

/* --- @xrealloc@ --- *
 *
 * Arguments:	@void *p@ = pointer to a block of memory
 *		@size_t sz@ = new size desired for the block
 *
 * Returns:	Pointer to the resized memory block (which is almost
 *		certainly not in the same place any more).
 *
 * Use:		Resizes a memory block.  If there's not enough memory, the
 *		exception @EXC_NOMEM@ is thrown.
 */

void *xrealloc(void *p, size_t sz)
{
  p = realloc(p, sz);
  if (!p)
    THROW(EXC_NOMEM);
  return (p);
}

/*----- That's all, folks -------------------------------------------------*/
