/* -*-c-*-
 *
 * $Id: alloc.h,v 1.3 1999/05/06 19:51:35 mdw Exp $
 *
 * Memory allocation functions
 *
 * (c) 1998 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the mLib utilities library.
 *
 * mLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * mLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with mLib; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------*
 *
 * $Log: alloc.h,v $
 * Revision 1.3  1999/05/06 19:51:35  mdw
 * Reformatted the LGPL notice a little bit.
 *
 * Revision 1.2  1999/05/05 18:50:31  mdw
 * Change licensing conditions to LGPL.
 *
 * Revision 1.1.1.1  1998/06/17 23:44:42  mdw
 * Initial version of mLib
 *
 */

#ifndef ALLOC_H
#define ALLOC_H

#ifdef __cplusplus
  extern "C" {
#endif

#include <stddef.h>

/*----- Functions provided ------------------------------------------------*/

/* --- @xmalloc@ --- *
 *
 * Arguments:	@size_t sz@ = size of block to allocate
 *
 * Returns:	Pointer to allocated block.
 *
 * Use:		Allocates memory.  If there's not enough memory, the
 *		exception @EXC_NOMEM@ is thrown.
 */

extern void *xmalloc(size_t /*sz*/);

/* --- @xstrdup@ --- *
 *
 * Arguments:	@const char *s@ = pointer to a string
 *
 * Returns:	Pointer to a copy of the string.
 *
 * Use:		Copies a string (like @strdup@ would, if it existed).  If
 *		there's not enough memory, the exception @EXC_NOMEM@ is
 *		thrown.
 */

extern char *xstrdup(const char */*s*/);

/* --- @xrealloc@ --- *
 *
 * Arguments:	@void *p@ = pointer to a block of memory
 *		@size_t sz@ = new size desired for the block
 *
 * Returns:	Pointer to the resized memory block (which is almost
 *		certainly not in the same place any more).
 *
 * Use:		Resizes a memory block.  If there's not enough memory, the
 *		exception @EXC_NOMEM@ is thrown.
 */

extern void *xrealloc(void */*p*/, size_t /*sz*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
