/* -*-c-*-
 *
 * $Id: pquis.c,v 1.1 1999/10/04 21:43:29 mdw Exp $
 *
 * Print strings, substituting the program name
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the mLib utilities library.
 *
 * mLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * mLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with mLib; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: pquis.c,v $
 * Revision 1.1  1999/10/04 21:43:29  mdw
 * New function `pquis'.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include "quis.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @pquis@ --- *
 *
 * Arguments:	@FILE *fp@ = output stream to write on
 *		@const char *p@ = pointer to string to write
 *
 * Returns:	Zero if everything worked, EOF if not.
 *
 * Use:		Writes the string @p@ to the output stream @fp@.  Occurrences
 *		of the character `$' in @p@ are replaced by the program name
 *		as reported by @quis@.  A `$$' is replaced by a single `$'
 *		sign.
 */

int pquis(FILE *fp, const char *p)
{
  size_t sz;

  while (*p) {
    sz = strcspn(p, "$");
    if (sz) {
      if (fwrite(p, 1, sz, fp) < sz)
	return (EOF);
      p += sz;
    }
    if (*p == '$') {
      p++;
      if (*p == '$') {
	if (fputc('$', fp) == EOF)
	  return (EOF);
	p++;
      } else {
	if (fputs(pn__name, fp) == EOF)
	  return (EOF);
      }
    }
  }
  return (0);
}

/*----- That's all, folks -------------------------------------------------*/
