/* -*-c-*-
 *
 * $Id: str.c,v 1.2 1999/05/26 20:52:57 mdw Exp $
 *
 * Functions for hacking with strings
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the mLib utilities library.
 *
 * mLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * mLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with mLib; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: str.c,v $
 * Revision 1.2  1999/05/26 20:52:57  mdw
 * Add new `rest' argument for `str_split'.
 *
 * Revision 1.1  1999/05/17 20:37:01  mdw
 * Some trivial string hacks.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "str.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @str_getword@ --- *
 *
 * Arguments:	@char **pp@ = address of pointer into string
 *
 * Returns:	Pointer to the next space-separated word from the string,
 *		or null.
 *
 * Use:		Parses off space-separated words from a string.
 */

char *str_getword(char **pp)
{
  char *p = *pp, *q;

  if (!p)
    return (0);

  while (isspace((unsigned char)*p))
    p++;

  for (q = p; *q; q++) {
    if (isspace((unsigned char)*q)) {
      *q = 0;
      *pp = q + 1;
      return (p);
    }
  }

  *pp = 0;
  return (p);
}

/* --- @str_split@ --- *
 *
 * Arguments:	@char *p@ = pointer to string
 *		@char *v[]@ = pointer to array to fill in
 *		@size_t c@ = count of strings to fill in
 *		@char **rest@ = where to store the remainder of the string
 *
 * Returns:	Number of strings filled in.
 *
 * Use:		Fills an array with pointers to the individual words of a
 *		string.  The string is modified in place to contain zero
 *		bytes at the word boundaries, and the words have leading
 *		and trailing space stripped off.  No more than @c@ words
 *		are read; the actual number is returned as the value of the
 *		function.  Unused slots in the array are populated with
 *		null bytes.  If there's any string left, the address of the
 *		remainder is stored in @rest@ (if it's non-null); otherwise
 *		@rest@ is set to a null pointer.
 */

size_t str_split(char *p, char *v[], size_t c, char **rest)
{
  size_t n = 0;
  char *q;

  while (c && (q = str_getword(&p)) != 0) {
    *v++ = q;
    c--;
    n++;
  }

  while (c) {
    *v++ = 0;
    c--;
  }

  if (rest) {
    if (!p)
      *rest = 0;
    else {
      while (isspace((unsigned char)*p))
	p++;
      if (*p)
	*rest = p;
      else
	*rest = 0;
    }
  }
  return (n);
}

/* --- @str_sanitize@ --- *
 *
 * Arguments:	@char *d@ = destination buffer
 *		@const char *p@ = pointer to source string
 *		@size_t sz@ = size of destination buffer
 *
 * Returns:	---
 *
 * Use:		Writes a string into a buffer, being careful not to overflow
 *		the buffer, to null terminate the result, and to prevent
 *		nasty nonprintable characters ending up in the buffer.
 */

void str_sanitize(char *d, const char *p, size_t sz)
{
  if (!sz)
    return;
  sz--;
  while (*p && sz) {
    int ch = *p++;
    if (!isgraph((unsigned char)ch))
      ch = '_';
    *d++ = ch;
    sz--;
  }
  *d++ = 0;
}

/*----- That's all, folks -------------------------------------------------*/
