/* -*-c-*-
 *
 * $Id: sub.h,v 1.4 1999/05/13 22:48:55 mdw Exp $
 *
 * Allocation of known-size blocks
 *
 * (c) 1998 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the mLib utilities library.
 *
 * mLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * mLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with mLib; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------*
 *
 * $Log: sub.h,v $
 * Revision 1.4  1999/05/13 22:48:55  mdw
 * Change `-ise' to `-ize' throughout.
 *
 * Revision 1.3  1999/05/06 19:51:35  mdw
 * Reformatted the LGPL notice a little bit.
 *
 * Revision 1.2  1999/05/05 18:50:31  mdw
 * Change licensing conditions to LGPL.
 *
 * Revision 1.1.1.1  1998/06/17 23:44:42  mdw
 * Initial version of mLib
 *
 */

#ifndef SUB_H
#define SUB_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Required header files ---------------------------------------------*/

#include <stdlib.h>

#ifndef ALLOC_H
#  include "alloc.h"
#endif

/*----- Functions provided ------------------------------------------------*/

/* --- @sub_alloc@ --- *
 *
 * Arguments:   @size_t s@ = size of chunk wanted
 *
 * Returns:     Pointer to a block at least as large as the one wanted.
 *
 * Use:         Allocates a small block of memory.  If there is no more
 *		memory left, the exception @EXC_NOMEM@ is raised.
 */

#ifdef TRACK_ENABLE
#  define sub_alloc(s) xmalloc(s)
#else
void *sub_alloc(size_t s);
#endif

/* --- @sub_free@ --- *
 *
 * Arguments:   @void *p@ = address of block to free
 *              @size_t s@ = size of block
 *
 * Returns:     ---
 *
 * Use:         Frees a block allocated by @sub_alloc@.
 */

#ifdef TRACK_ENABLE
#  define sub_free(p, s) free(p)
#else
void sub_free(void *p, size_t s);
#endif

/* --- @CREATE@ --- *
 *
 * Arguments:	@type@ = type of object required; must be passable to
 *			@sizeof@
 *
 * Returns:	Pointer to a block sufficiently big to hold an object of the
 *		named type.
 *
 * Use:		Allocates a block of the required type.
 */

#define CREATE(type) sub_alloc(sizeof(type))

/* --- @DESTROY@ --- *
 *
 * Arguments:	@void *p@ = pointer to an object
 *
 * Returns:	---
 *
 * Use:		Frees the thing pointed to by @p@.
 */

#define DESTROY(p) sub_free(p, sizeof(*p))

/* --- @sub_init@ --- *
 *
 * Arguments:   ---
 *
 * Returns:     ---
 *
 * Use:         Initializes the magic allocator.
 */

void sub_init(void);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
