/* -*-c-*-
 *
 * $Id: testrig.h,v 1.4 1999/07/06 19:15:28 mdw Exp $
 *
 * Generic test driver
 *
 * (c) 1998 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the mLib utilities library.
 *
 * mLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * mLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with mLib; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: testrig.h,v $
 * Revision 1.4  1999/07/06 19:15:28  mdw
 * Comment out argument in structure definition.
 *
 * Revision 1.3  1999/05/06 19:51:35  mdw
 * Reformatted the LGPL notice a little bit.
 *
 * Revision 1.2  1999/05/05 18:50:31  mdw
 * Change licensing conditions to LGPL.
 *
 * Revision 1.1.1.1  1998/06/17 23:44:42  mdw
 * Initial version of mLib
 *
 */

#ifndef TESTER_H
#define TESTER_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <stddef.h>

#include "dstr.h"

/*----- Magical numbers ---------------------------------------------------*/

#define TEST_FIELDMAX 16		/* Maximum fields in a line */

/*----- Data structures ---------------------------------------------------*/

/* --- Test field definition --- */

typedef struct test_type {
  void (*cvt)(const char *buf, dstr *d); /* Conversion function */
  void (*dump)(dstr *d, FILE *fp);	/* Dump function */
} test_type;

/* --- Test chunk definition --- */

typedef struct test_chunk {
  const char *name;			/* Name of this chunk */
  int (*test)(dstr /*dv*/[]);		/* Test verification function */
  test_type *f[TEST_FIELDMAX];		/* Field definitions */
} test_chunk;

/*----- Predefined data types ---------------------------------------------*/

extern test_type type_hex;
extern test_type type_string;
extern test_type type_int;

/*----- Functions provided ------------------------------------------------*/

/* --- @test_run@ --- *
 *
 * Arguments:	@int argc@ = number of command line arguments
 *		@char *argv[]@ = pointer to command line arguments
 *		@const test_chunk chunk[]@ = pointer to chunk definitions
 *		@const char *def@ = name of default test vector file
 *
 * Returns:	Doesn't.
 *
 * Use:		Runs a set of test vectors to ensure that a component is
 *		working properly.
 */

extern void test_run(int /*argc*/, char */*argv*/[],
		     const test_chunk /*chunk*/[],
		     const char */*def*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
